@@ -3,6 +3,7 @@
 import importlib
 import json
 import os
+import re
 import shutil
 import sys
 import warnings
@@ -13,7 +14,7 @@ from io import BytesIO
 from pathlib import Path
 from types import ModuleType
 from typing import Any, Dict, Iterator, KeysView, List, Optional, Set, Tuple, Union
-from urllib.parse import urlparse
+from urllib.parse import quote, urlparse
 
 import requests
 import yaml
@@ -34,6 +35,7 @@ from brownie._config import (
 )
 from brownie._expansion import expand_posix_vars
 from brownie.exceptions import (
+    BadProjectName,
     BrownieEnvironmentWarning,
     InvalidPackage,
     PragmaError,
@@ -58,6 +60,7 @@ BUILD_FOLDERS = ["contracts", "deployments", "interfaces"]
 MIXES_URL = "https://github.com/brownie-mix/{}-mix/archive/{}.zip"
 
 GITIGNORE = """__pycache__
+.env
 .history
 .hypothesis/
 build/
@@ -89,13 +92,18 @@ class _ProjectBase:
             os.chdir(self._path)
 
         try:
+            project_evm_version = compiler_config["evm_version"]
+            evm_version = {
+                "Solidity": compiler_config["solc"].get("evm_version", project_evm_version),
+                "Vyper": compiler_config["vyper"].get("evm_version", project_evm_version),
+            }
             build_json = compiler.compile_and_format(
                 contract_sources,
                 solc_version=compiler_config["solc"].get("version", None),
                 vyper_version=compiler_config["vyper"].get("version", None),
                 optimize=compiler_config["solc"].get("optimize", None),
                 runs=compiler_config["solc"].get("runs", None),
-                evm_version=compiler_config["evm_version"],
+                evm_version=evm_version,
                 silent=silent,
                 allow_paths=allow_paths,
                 remappings=compiler_config["solc"].get("remappings", []),
@@ -179,11 +187,13 @@ class Project(_ProjectBase):
         self._active = False
         self.load()
 
-    def load(self) -> None:
+    def load(self, raise_if_loaded: bool = True) -> None:
         """Compiles the project contracts, creates ContractContainer objects and
         populates the namespace."""
         if self._active:
-            raise ProjectAlreadyLoaded("Project is already active")
+            if raise_if_loaded:
+                raise ProjectAlreadyLoaded("Project is already active")
+            return None
 
         contract_sources = _load_sources(self._path, self._structure["contracts"], False)
         interface_sources = _load_sources(self._path, self._structure["interfaces"], True)
@@ -191,15 +201,19 @@ class Project(_ProjectBase):
         self._build = Build(self._sources)
 
         contract_list = self._sources.get_contract_list()
+        potential_dependencies = []
         for path in list(self._build_path.glob("contracts/*.json")):
             try:
                 with path.open() as fp:
                     build_json = json.load(fp)
             except json.JSONDecodeError:
                 build_json = {}
-            if not set(BUILD_KEYS).issubset(build_json) or path.stem not in contract_list:
+            if not set(BUILD_KEYS).issubset(build_json):
                 path.unlink()
                 continue
+            if path.stem not in contract_list:
+                potential_dependencies.append((path, build_json))
+                continue
             if isinstance(build_json["allSourcePaths"], list):
                 # this handles the format change in v1.7.0, it can be removed in a future release
                 path.unlink()
@@ -212,6 +226,14 @@ class Project(_ProjectBase):
                 continue
             self._build._add_contract(build_json)
 
+        for path, build_json in potential_dependencies:
+            dependents = self._build.get_dependents(path.stem)
+            is_dependency = len(set(dependents) & set(contract_list)) > 0
+            if is_dependency:
+                self._build._add_contract(build_json)
+            else:
+                path.unlink()
+
         interface_hashes = {}
         interface_list = self._sources.get_interface_list()
         for path in list(self._build_path.glob("interfaces/*.json")):
@@ -288,17 +310,19 @@ class Project(_ProjectBase):
         if build_json["sha1"] != sha1(source.encode()).hexdigest():
             return True
         # compare compiler settings
-        if _compare_settings(config, build_json["compiler"]):
-            return True
         if build_json["language"] == "Solidity":
             # compare solc-specific compiler settings
             solc_config = config["solc"].copy()
             solc_config["remappings"] = None
-            if _compare_settings(solc_config, build_json["compiler"]):
+            if not _solidity_compiler_equal(solc_config, build_json["compiler"]):
                 return True
             # compare solc pragma against compiled version
             if Version(build_json["compiler"]["version"]) not in get_pragma_spec(source):
                 return True
+        else:
+            vyper_config = config["vyper"].copy()
+            if not _vyper_compiler_equal(vyper_config, build_json["compiler"]):
+                return True
         return False
 
     def _compile_interfaces(self, compiled_hashes: Dict) -> None:
@@ -315,6 +339,7 @@ class Project(_ProjectBase):
         changed_sources = {i: self._sources.get(i) for i in changed_paths}
         abi_json = compiler.get_abi(
             changed_sources,
+            solc_version=self._compiler_config["solc"].get("version", None),
             allow_paths=self._path.as_posix(),
             remappings=self._compiler_config["solc"].get("remappings", []),
         )
@@ -696,7 +721,11 @@ def compile_source(
         raise exc
 
 
-def load(project_path: Union[Path, str, None] = None, name: Optional[str] = None) -> "Project":
+def load(
+    project_path: Union[Path, str, None] = None,
+    name: Optional[str] = None,
+    raise_if_loaded: bool = True,
+) -> "Project":
     """Loads a project and instantiates various related objects.
 
     Args:
@@ -733,9 +762,15 @@ def load(project_path: Union[Path, str, None] = None, name: Optional[str] = None
         name = project_path.name
         if not name.lower().endswith("project"):
             name += " project"
-        name = "".join(i for i in name.title() if i.isalpha())
-    if next((True for i in _loaded_projects if i._name == name), False):
-        raise ProjectAlreadyLoaded("There is already a project loaded with this name")
+        if not name[0].isalpha():
+            raise BadProjectName("Project must start with an alphabetic character")
+        name = "".join(i for i in name.title() if i.isalnum())
+
+    for loaded_project in _loaded_projects:
+        if loaded_project._name == name:
+            if raise_if_loaded:
+                raise ProjectAlreadyLoaded("There is already a project loaded with this name")
+            return loaded_project
 
     # paths
     _create_folders(project_path)
@@ -783,11 +818,12 @@ def _install_from_ethpm(uri: str) -> str:
     install_path.mkdir(exist_ok=True)
     install_path = install_path.joinpath(f"{repo}@{version}")
     if install_path.exists():
-        raise FileExistsError("Package is aleady installed")
+        raise FileExistsError("Package is already installed")
 
     try:
         new(str(install_path), ignore_existing=True)
         ethpm.install_package(install_path, uri)
+        Path.touch(install_path / ".env")
         project = load(install_path)
         project.close()
     except Exception as e:
@@ -811,7 +847,7 @@ def _maybe_retrieve_github_auth() -> Dict[str, str]:
 
 def _install_from_github(package_id: str) -> str:
     try:
-        path, version = package_id.split("@")
+        path, version = package_id.split("@", 1)
         org, repo = path.split("/")
     except ValueError:
         raise ValueError(
@@ -829,36 +865,24 @@ def _install_from_github(package_id: str) -> str:
     headers = REQUEST_HEADERS.copy()
     headers.update(_maybe_retrieve_github_auth())
 
-    response = requests.get(
-        f"https://api.github.com/repos/{org}/{repo}/tags?per_page=100", headers=headers
-    )
-    if response.status_code != 200:
-        msg = "Status {} when getting package versions from Github: '{}'".format(
-            response.status_code, response.json()["message"]
-        )
-        if response.status_code in (403, 404):
-            msg += (
-                "\n\nMissing or forbidden.\n"
-                "If this issue persists, generate a Github API token and store"
-                " it as the environment variable `GITHUB_TOKEN`:\n"
-                "https://github.blog/2013-05-16-personal-api-tokens/"
-            )
-        raise ConnectionError(msg)
-
-    data = response.json()
-    if not data:
-        raise ValueError("Github repository has no tags set")
-    org, repo = data[0]["zipball_url"].split("/")[3:5]
-    tags = [i["name"].lstrip("v") for i in data]
-    if version not in tags:
-        raise ValueError(
-            "Invalid version for this package. Available versions are:\n" + ", ".join(tags)
-        ) from None
-
-    download_url = next(i["zipball_url"] for i in data if i["name"].lstrip("v") == version)
+    if re.match(r"^[0-9a-f]+$", version):
+        download_url = f"https://api.github.com/repos/{org}/{repo}/zipball/{version}"
+    else:
+        download_url = _get_download_url_from_tag(org, repo, version, headers)
 
     existing = list(install_path.parent.iterdir())
-    _stream_download(download_url, str(install_path.parent), headers)
+
+    # Some versions contain special characters and github api seems to display url without
+    # encoding them.
+    # It results in a ConnectionError exception because the actual download url is encoded.
+    # In this case we try to sanitize the version in url and download again.
+    try:
+        _stream_download(download_url, str(install_path.parent), headers)
+    except ConnectionError:
+        download_url = (
+            f"https://api.github.com/repos/{org}/{repo}/zipball/refs/tags/{quote(version)}"
+        )
+        _stream_download(download_url, str(install_path.parent), headers)
 
     installed = next(i for i in install_path.parent.iterdir() if i not in existing)
     shutil.move(installed, install_path)
@@ -888,6 +912,8 @@ def _install_from_github(package_id: str) -> str:
             with install_path.joinpath("brownie-config.yaml").open("w") as fp:
                 yaml.dump(brownie_config, fp)
 
+            Path.touch(install_path / ".env")
+
         project = load(install_path)
         project.close()
     except InvalidPackage:
@@ -903,6 +929,36 @@ def _install_from_github(package_id: str) -> str:
     return f"{org}/{repo}@{version}"
 
 
+def _get_download_url_from_tag(org: str, repo: str, version: str, headers: dict) -> str:
+    response = requests.get(
+        f"https://api.github.com/repos/{org}/{repo}/tags?per_page=100", headers=headers
+    )
+    if response.status_code != 200:
+        msg = "Status {} when getting package versions from Github: '{}'".format(
+            response.status_code, response.json()["message"]
+        )
+        if response.status_code in (403, 404):
+            msg += (
+                "\n\nMissing or forbidden.\n"
+                "If this issue persists, generate a Github API token and store"
+                " it as the environment variable `GITHUB_TOKEN`:\n"
+                "https://github.blog/2013-05-16-personal-api-tokens/"
+            )
+        raise ConnectionError(msg)
+
+    data = response.json()
+    if not data:
+        raise ValueError("Github repository has no tags set")
+    org, repo = data[0]["zipball_url"].split("/")[3:5]
+    tags = [i["name"].lstrip("v") for i in data]
+    if version not in tags:
+        raise ValueError(
+            "Invalid version for this package. Available versions are:\n" + ", ".join(tags)
+        ) from None
+
+    return next(i["zipball_url"] for i in data if i["name"].lstrip("v") == version)
+
+
 def _create_gitfiles(project_path: Path) -> None:
     gitignore = project_path.joinpath(".gitignore")
     if not gitignore.exists():
@@ -937,6 +993,22 @@ def _compare_settings(left: Dict, right: Dict) -> bool:
     )
 
 
+def _normalize_solidity_version(version: str) -> str:
+    return version.split("+")[0]
+
+
+def _solidity_compiler_equal(config: dict, build: dict) -> bool:
+    return (
+        config["version"] is None
+        or _normalize_solidity_version(config["version"])
+        == _normalize_solidity_version(build["version"])
+    ) and config["optimizer"] == build["optimizer"]
+
+
+def _vyper_compiler_equal(config: dict, build: dict) -> bool:
+    return config["version"] is None or config["version"] == build["version"]
+
+
 def _load_sources(project_path: Path, subfolder: str, allow_json: bool) -> Dict:
     contract_sources: Dict = {}
     suffixes: Tuple = (".sol", ".vy")
